#property strict

#include "Pointer.mqh"

#define SIMPLE_COMPARE_MACROS(NAME, SOURCE, VALUE)                    \
  class NAME : public INTERFACE_COMPARE                               \
  {                                                                   \
  private:                                                            \
    double Condition( const SOURCE &Source ) const                    \
    {                                                                 \
      return((double)(VALUE));                                        \
    }                                                                 \
  public:                                                             \
    double Compare( const SOURCE &First, const SOURCE &Second ) const \
    {                                                                 \
      return(SetMode(Condition(First) - Condition(Second)));          \
    }                                                                 \
                                                                      \
    NAME( const int iMode = MODE_ASCEND ) : INTERFACE_COMPARE(iMode)  \
    {                                                                 \
    }                                                                 \
  };                                                                  \

class INTERFACE_COMPARE
{
private:
  const int Mode;

protected:
  template <typename T>
  T SetMode( T Value ) const
  {
    if (Mode != MODE_ASCEND)
      Value = -Value;

    return(Value);
  }

public:
  INTERFACE_COMPARE( const int iMode = MODE_ASCEND ): Mode(iMode)
  {
  }

  template<typename T>
  T Compare( const T First, const T Second ) const
  {
    return(SetMode(First - Second));
  }
};

class SORT
{
private:
  template <typename T>
  static void Swap( T &Array[], const int i, const int j )
  {
    const T Temp = Array[i];

    Array[i] = Array[j];
    Array[j] = Temp;

    return;
  }

  template<typename T1, typename T2>
  static int Partition( T1 &Array[], const T2 &Compare, const int Start, const int End )
  {
    int Marker = Start;

    for (int i = Start; i <= End; i++)
      if (Compare.Compare(Array[i], Array[End]) <= 0)
      {
        Swap(Array, i, Marker);

        Marker++;
      }

     return(Marker - 1);
  }

  template<typename T1, typename T2>
  static void QuickSort( T1 &Array[], const T2 &Compare, const int Start, const int End )
  {
    if (Start < End)
    {
      const int Pivot = Partition(Array, Compare, Start, End);

      QuickSort(Array, Compare, Start, Pivot - 1);
      QuickSort(Array, Compare, Pivot + 1, End);
    }

    return;
  }

public:
  /*    MQL- ArraySort:
       void&       array[]                   //   
       void&       Compare                   //   ( )
       int         count = WHOLE_ARRAY,      //  
       int         start = 0,                //  
  */
  template<typename T1, typename T2>
  static void Sort( T1 &Array[], const T2 Compare, int Count = WHOLE_ARRAY, const int Start = 0 )
  {
    if (Count == WHOLE_ARRAY)
      Count = ArraySize(Array);

    if (CheckPointer(Compare) != POINTER_INVALID)
      QuickSort(Array, Compare, Start, Start + Count - 1);

    POINTER::DeleteObject(Compare);

    return;
  }
};